==> ppmbutton-2.1.0.tar.gz: ppmbutton-2.1.0/roxenfont.c <==
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include <getopt.h>		/* for getopt_long */

#include <ant/string.h>
#include <ant/io.h>
#include <ant/alloc.h>

#include "pixmap.h"
#include "spline.h"
#include "x11.h"

char *cmnd;		/* name this program was invoked by */


void usage(void)
{
    fprintf(stderr, "Usage: %s "
	    "{--truetype|--x11|-f} fontname "
	    "[{--em|-M} pixels] "
	    "[{--scale|-s} factor] "
	    "[{--output|-o} filename] "
	    "\n",
	    cmnd);
    exit(1);
};


pixmapT *bitmap2graymap(
		      pixmapT * bitmap,
		      int scale
)
{
    pixmapT *pm = alloc_pixmap((bitmap->rows + (scale - 1)) / scale, (bitmap->cols + (scale - 1)) / scale);
    int i, j;

    for (i = 0; i < pm->rows; i++) {
	for (j = 0; j < pm->cols; j++) {
	    int sum = 0;
	    int m;
	    for (m = 0; m < scale; m++) {
		int n;
		for (n = 0; n < scale; n++) {
		    if (i * scale + m < bitmap->rows &&
		        j * scale + n < bitmap->cols && 
	                bitmap->pixels[i * scale + m][j * scale + n]) {
			    sum ++;
		    }
		}
	    }
	    sum = sum * 255 / (scale * scale);
	    pm->pixels[i][j] = pixel(sum, 255-sum, 255-sum, 255-sum);
	}
    }
    return pm;
}



void dumpfont(
    const char *outfilename,
    int numchars,
    int numglyphs,
    int baseline,
    int *idx,
    pixmapT **pixmaps
){
    FILE *fp;
    int i;
    long off;
    long *offsets;
    long rows = pixmaps[1]->rows;
    
    if (STREQ(outfilename, "-")) {
	fp = stdout;
    } else {
	fp = efopen(outfilename, "wb");
    }
    writeulongbe(0x464f4e54, fp);
    writeulongbe(1, fp);
    writeulongbe(numchars, fp);
    writeulongbe(rows, fp);
    writeulongbe(baseline, fp);
    off = (5 + numchars) * 4;
    offsets = emalloc(numchars * sizeof(*offsets));
    fprintf(stderr, "numglyphs=%d\n", numglyphs);
    fprintf(stderr, "numchars=%d\n", numchars);
    for (i = 0; i < numglyphs; i++) {
	pixmapT *pm = pixmaps[i];
        int sz = rows * pm->cols;
	offsets[i] = off;
	off += (sz + 3) / 4 * 4 + 8;
    }
    for (i = 0; i < numchars; i++) {
	int j;
	j = idx[i];
	off = offsets[j];
	writeulongbe(off, fp);
    }
    for (i = 0; i < numglyphs; i++) {
	pixmapT *pm = pixmaps[i];
	int j, k;
	off = ftell(fp);
	assert(offsets[i] - 3 <= off && off <= offsets[i]);
	for (; off < offsets[i]; off++) {
	    putc(0, fp);
	}
	writeulongbe(pm->cols, fp);
	writeulongbe(pm->cols, fp);
	for (j = 0; j < rows; j++) {
	    for (k = 0; k < pm->cols; k++) {
		putc(green(pm->pixels[j][k]), fp);
	    }
	}
    }
}


int main(int argc, char *argv[])
{


    char *fn = NULL;
    int pixelsPerEm = 32;
    pixmapT *bitmap = NULL;

    static struct option o[] =
    {
	{"truetype", 1, 0, 'f'},
	{"x11", 1, 0, 'f'},
	{"em", 1, 0, 'M'},
	{"scale", 1, 0, 's'},
	{"output", 1, 0, 'o'},
	{NULL, 0, 0, 0}
    };
    int c;
    int longind;
    int use_truetype = 0;
    int use_x11 = 0;
    char *p;
    char *outfilename = "-";
    int scale = 1;
    pixmapT **pixmaps = NULL;
    int *idx = NULL;
    int numchars = 0;
    int numglyphs = 0;
    int baseline = 0;

    cmnd = argv[0];

    while ((c = getopt_long(argc, argv, "f:o:s:M:", o, &longind)) != EOF) {
	switch (c) {
	case 0:
	    fn = optarg;
	    switch (o[longind].name[0]) {
	    case 't':
		use_truetype = 1;
		break;
	    case 'x':
		use_x11 = 1;
		break;
	    default:
		printf("%s?\n", o[longind].name);
                usage();
	    }
	    break;

	case 's':
	    scale = strtol(optarg, &p, 0);
	    if (p == optarg || *p)
		usage();
	    break;

	case 'f':
	    fn = optarg;
	    if (strlen(fn) > 4 && strcmp(fn + strlen(fn) - 4, ".ttf") == 0) {
		use_truetype = 1;
	    } else {
		use_x11 = 1;
	    }
	    break;

	case 'M':
	    pixelsPerEm = strtol(optarg, &p, 0);
	    if (p == optarg || *p)
		usage();
	    break;

	case 'o':
	    outfilename = optarg;
	    break;

	}
    }

    if (use_truetype) {
        char *charmap = ttfCharMap(fn);
	int i, j;
	j = 1;
	idx = ecalloc(65536, sizeof(*idx));
	pixmaps = emalloc(65536 * sizeof(*pixmaps));
	pixmaps[0] = alloc_pixmap(0, 0);
	for (i = 0; i < 65536; i++) {
	    if (charmap[i] & 1) {
		char text[2+5+1+1];
		if (i < 256 && i != '&') {
		    sprintf(text, "%c", i);
		} else {
		    sprintf(text, "&#%d;", i);
		}
		bitmap = ttfRenderString(fn, text, pixelsPerEm); 
		pixmaps[j] = bitmap2graymap(bitmap, scale);
		idx[i] = j++;
	    } else {
		idx[i] = 0;
	    }
	    if (charmap[i] & 2) break;
	}
	numchars = i + 1;
	numglyphs = j;
    } else if (use_x11) {
	char text[2+5+1+1];
	assert(0); /* NYI */
	bitmap = x11RenderString(fn, text);
    } else {
	usage();
    }
    dumpfont(outfilename, numchars, numglyphs, baseline, idx, pixmaps);

    return 0;
}

==> ppmbutton-2.1.4.tar.gz: ppmbutton-2.1.4/roxenfont.c <==
#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#include <ant/getopt.h>		/* for getopt_long */

#include <ant/string.h>
#include <ant/io.h>
#include <ant/alloc.h>

#include "pixmap.h"
#include "spline.h"
#include "x11.h"
#include "ttf.h"

char *cmnd;		/* name this program was invoked by */


static void usage(void)
{
    fprintf(stderr, "Usage: %s "
	    "{--truetype|--x11|-f} fontname "
	    "[{--em|-M} pixels] "
	    "[{--scale|-s} factor] "
	    "[{--output|-o} filename] "
	    "[{--id|-i} platformId:specificId] "
	    "\n",
	    cmnd);
    exit(1);
}


static pixmapT *bitmap2graymap(
		      pixmapT * bitmap,
		      int scale
)
{
    pixmapT *pm = alloc_pixmap((bitmap->rows + (scale - 1)) / scale, (bitmap->cols + (scale - 1)) / scale);
    int i, j;

    for (i = 0; i < pm->rows; i++) {
	for (j = 0; j < pm->cols; j++) {
	    int sum = 0;
	    int m;
	    for (m = 0; m < scale; m++) {
		int n;
		for (n = 0; n < scale; n++) {
		    if (i * scale + m < bitmap->rows &&
		        j * scale + n < bitmap->cols && 
	                bitmap->pixels[i * scale + m][j * scale + n]) {
			    sum ++;
		    }
		}
	    }
	    sum = sum * 255 / (scale * scale);
	    pm->pixels[i][j] = pixel(sum, 255-sum, 255-sum, 255-sum);
	}
    }
    return pm;
}



static void dumpfont(
    const char *outfilename,
    int numchars,
    int numglyphs,
    int baseline,
    int *idx,
    pixmapT **pixmaps
){
    FILE *fp;
    int i;
    long off;
    long *offsets;
    long rows = pixmaps[1]->rows;
    
    if (STREQ(outfilename, "-")) {
	fp = stdout;
    } else {
	fp = efopen(outfilename, "wb");
    }
    writeulongbe(0x464f4e54, fp);
    writeulongbe(1, fp);
    writeulongbe(numchars, fp);
    writeulongbe(rows, fp);
    writeulongbe(baseline, fp);
    off = (5 + numchars) * 4;
    offsets = emalloc(numchars * sizeof(*offsets));
    fprintf(stderr, "numglyphs=%d\n", numglyphs);
    fprintf(stderr, "numchars=%d\n", numchars);
    for (i = 0; i < numglyphs; i++) {
	pixmapT *pm = pixmaps[i];
        int sz = rows * pm->cols;
	offsets[i] = off;
	off += (sz + 3) / 4 * 4 + 8;
    }
    for (i = 0; i < numchars; i++) {
	int j;
	j = idx[i];
	off = offsets[j];
	writeulongbe(off, fp);
    }
    for (i = 0; i < numglyphs; i++) {
	pixmapT *pm = pixmaps[i];
	int j, k;
	off = ftell(fp);
	assert(offsets[i] - 3 <= off && off <= offsets[i]);
	for (; off < offsets[i]; off++) {
	    putc(0, fp);
	}
	writeulongbe(pm->cols, fp);
	writeulongbe(pm->cols, fp);
	for (j = 0; j < rows; j++) {
	    for (k = 0; k < pm->cols; k++) {
		putc(green(pm->pixels[j][k]), fp);
	    }
	}
    }
}


int main(int argc, char *argv[])
{


    char *fn = NULL;
    int pixelsPerEm = 32;
    pixmapT *bitmap = NULL;

    static struct option o[] =
    {
	{"truetype", 1, 0, 'f'},
	{"x11", 1, 0, 'f'},
	{"em", 1, 0, 'M'},
	{"scale", 1, 0, 's'},
	{"output", 1, 0, 'o'},
	{"id", 1, 0, 'i'},
	{NULL, 0, 0, 0}
    };
    int c;
    int longind;
    int use_truetype = 0;
    int use_x11 = 0;
    char *p;
    char *outfilename = "-";
    int scale = 1;
    pixmapT **pixmaps = NULL;
    int *idx = NULL;
    int numchars = 0;
    int numglyphs = 0;
    int baseline = 0;
    int platformID = 3;	/* Windows */
    int specificID = 1;	/* Unicode */
     

    cmnd = argv[0];

    while ((c = getopt_long(argc, argv, "f:o:s:M:i:", o, &longind)) != EOF) {
	switch (c) {
	case 0:
	    fn = optarg;
	    switch (o[longind].name[0]) {
	    case 't':
		use_truetype = 1;
		break;
	    case 'x':
		use_x11 = 1;
		break;
	    default:
		printf("%s?\n", o[longind].name);
                usage();
	    }
	    break;

	case 's':
	    scale = strtol(optarg, &p, 0);
	    if (p == optarg || *p)
		usage();
	    break;

	case 'f':
	    fn = optarg;
	    if (strlen(fn) > 4 && strcmp(fn + strlen(fn) - 4, ".ttf") == 0) {
		use_truetype = 1;
	    } else {
		use_x11 = 1;
	    }
	    break;

	case 'M':
	    pixelsPerEm = strtol(optarg, &p, 0);
	    if (p == optarg || *p)
		usage();
	    break;

	case 'o':
	    outfilename = optarg;
	    break;

	case 'i':
	    platformID = strtol(optarg, &p, 0);
	    if (p == optarg || *p != ':')
		usage();
	    specificID = strtol(p + 1, &p, 0);
	    if (p == optarg || *p != '\0')
		usage();
	    break;

	}
    }

    if (use_truetype) {
        char *charmap;
	int i, j;
	ttfHandle *handle;
	j = 1;

	/* Open a TrueType font file and parse the most important tables */
	if ((handle = ttfOpen(fn)) == NULL) {
	    fprintf(stderr, "%s: cannot open ttf File %s: %s\n",
	    	    cmnd, fn, strerror(errno));
	    exit(1);
	}
        if ((charmap = ttfCharMap(handle, platformID, specificID)) == NULL) {
	    fprintf(stderr, "%s: cannot open %s: %s\n",
	    	    cmnd, fn, strerror(errno));
	    exit(1);
        }
	idx = ecalloc(65536, sizeof(*idx));
	pixmaps = emalloc(65536 * sizeof(*pixmaps));
	pixmaps[0] = alloc_pixmap(0, 0);
	for (i = 0; i < 65536; i++) {
	    if (charmap[i] & 1) {
		char text[2+5+1+1];
		fprintf(stderr, "[%d] ", i);
		if (i < 256 && i != '&') {
		    sprintf(text, "%c", i);
		} else {
		    sprintf(text, "&#%d;", i);
		}
		bitmap = ttfRenderString(handle, text, pixelsPerEm,
					 platformID, specificID); 
		pixmaps[j] = bitmap2graymap(bitmap, scale);
		free_pixmap(bitmap);
		idx[i] = j++;
	    } else {
		idx[i] = 0;
	    }
	    if (charmap[i] & 2) break;
	}
	numchars = i + 1;
	numglyphs = j;
    } else if (use_x11) {
	char text[2+5+1+1];
	assert(0); /* NYI */
	bitmap = x11RenderString(fn, text);
    } else {
	usage();
    }
    dumpfont(outfilename, numchars, numglyphs, baseline, idx, pixmaps);

    return 0;
}
